import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';

import 'firebase/firebase_image.dart';
import 'info_dialog.dart';
import 'models/folder_model.dart';
import 'models/folder_model_firebase.dart';
import 'models/item_model.dart';
import 'models/item_model_firebase.dart';

class CardDetails extends StatefulWidget {

  final String? _editingId;
  final FolderModel? _folder;

  final TextEditingController _nameController = TextEditingController();
  final TextEditingController _noteController = TextEditingController();
  final String _cardName;
  final String? _existingImageUrl;
  final String _userCredential;
  XFile? _image;

  CardDetails({Key? key,
      String? editingId,
      String? nameController,
      String? noteController,
      FolderModel? folderModel,
      String? existingImageUrl,
      required String cardName,
      required String userCredential}) :
        _existingImageUrl = existingImageUrl,
        _editingId = editingId,
        _folder = folderModel,
        _cardName = cardName,
        _userCredential = userCredential,
        super(key: key) {
    if (nameController != null) {
      _nameController.text = nameController;
    }
    if (noteController != null) {
      _noteController.text = noteController;
    }
  }

  @override
  State<StatefulWidget> createState() => _CardDetailsState();
}

class _CardDetailsState extends State<CardDetails> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: AppBar(
        title: Text(
          widget._existingImageUrl == null
              ? 'Add ' + widget._cardName
              : widget._cardName + ' Detail',
        ),
      ),
      body: Column(
        mainAxisSize: MainAxisSize.max,
        children: [
          const SizedBox(height: 15.0),
          _buildTextField(widget._nameController, widget._cardName + ' name', 1),
          _buildTextField(widget._noteController, 'Note', 2),
          Padding(
            padding: const EdgeInsets.all(10.0),
            child: _buildTextImageButton(context),
          ),
          _buildImageField(),
          Padding(
            padding: const EdgeInsets.all(10.0),
            child: _buildTextButton(context),
          ),
        ],
      ),
    );
  }

  Widget _buildTextField(
      TextEditingController controller, String label, int maxLines) {
    return Padding(
      padding: const EdgeInsets.all(5.0),
      child: TextField(
        controller: controller,
        maxLines: maxLines,
        decoration: InputDecoration(
          label: Text(label),
          border: const OutlineInputBorder(),
        ),
      ),
    );
  }

  TextButton _buildTextButton(BuildContext context) {
    return TextButton(
      child: Text(
        widget._existingImageUrl == null
          ? 'Add ' + widget._cardName
          : 'Save ' + widget._cardName + ' Changes',
        style: const TextStyle(
          fontSize: 18.0,
          color: Colors.white,
        ),
      ),
      onPressed: () {
        if (widget._cardName == "Folder") {
          _submitFolder(context);
        } else {
          _submitItem(context);
        }
      },
      style: ButtonStyle(
        backgroundColor: MaterialStateProperty.all(Colors.blueAccent),
      ),
    );
  }

  TextButton _buildTextImageButton(BuildContext context) {
    return TextButton.icon(
      label: const Text(
        'Upload new image',
        style: TextStyle(
          fontSize: 18.0,
          color: Colors.white,
        ),
      ),
      onPressed: () async {
        var image = await getImageFromUser();
        setState(() {
          widget._image = image;
        });
      },
      style: ButtonStyle(
        backgroundColor: MaterialStateProperty.all(Colors.blueAccent),
      ),
      icon: const Icon(
        CupertinoIcons.arrow_up_doc,
        color: Colors.white,
        size: 23.0,
      ),
    );
  }

  Future<XFile?> getImageFromUser() async {
    return await ImagePicker().pickImage(source: ImageSource.gallery);
  }

  Future<void> _submitItem(BuildContext context) async {
    ItemModel item;
    if (widget._image == null && widget._existingImageUrl != null) {
      item = ItemModel(
        widget._editingId!,
        widget._userCredential,
        widget._folder!.id,
        widget._nameController.text,
        widget._existingImageUrl!,
        widget._noteController.text);
    } else if (widget._image != null) {
      final url =
        await FirebaseImage().uploadImageByPath(File(widget._image!.path));
      if (widget._editingId != null) {
        item = ItemModel(
          widget._editingId!,
          widget._userCredential,
          widget._folder!.id,
          widget._nameController.text,
          url,
          widget._noteController.text);
      } else {
        item = ItemModel.id(widget._userCredential, widget._folder!.id,
          widget._nameController.text, url, widget._noteController.text);
      }
    } else {
      _showMissingFieldDialog(context);
      return;
    }
    ItemModelFirebase.storeItem(item);
    Navigator.of(context).pop();
  }

  void _showMissingFieldDialog(BuildContext context) async {
    await showDialog<String>(
      context: context,
      builder: (BuildContext context) {
        return const InfoDialog(
          dialogContent: "Please fill all fields and try again",
          optionContent: "OK",
        );
      },
    );
  }

  Future<void> _submitFolder(BuildContext context) async {
    FolderModel folder;
    if (widget._image == null && widget._existingImageUrl != null) {
      folder = FolderModel(
        widget._editingId!,
        widget._userCredential,
        widget._nameController.text,
        widget._existingImageUrl!,
        widget._noteController.text);
    } else if (widget._image != null) {
      final url =
        await FirebaseImage().uploadImageByPath(File(widget._image!.path));
      if (widget._editingId != null) {
        folder = FolderModel(widget._editingId!, widget._userCredential,
          widget._nameController.text, url, widget._noteController.text);
      } else {
        folder = FolderModel.id(widget._userCredential,
          widget._nameController.text, url, widget._noteController.text);
      }
    } else {
      _showMissingFieldDialog(context);
      return;
    }
    FolderModelFirebase.storeFolder(folder);
    Navigator.of(context).pop();
  }

  Widget _buildImageField() {
    if (widget._image != null) {
      return Image.file(
        File(widget._image!.path),
        height: 200,
        width: 200,
        fit: BoxFit.contain,
      );
    } else if(widget._existingImageUrl != null) {
      return Image.network(
        widget._existingImageUrl!,
        height: 200,
        width: 200,
        fit: BoxFit.contain,
      );
    } else {
      return Image.asset(
        'assets/no_image.PNG',
        height: 200,
        width: 200,
        fit: BoxFit.contain,
      );
    }
  }
}
