import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:google_sign_in/google_sign_in.dart';

import 'firebase/firebase_handler.dart';
import 'friends_page.dart';
import 'grid_card.dart';
import 'models/folder_model.dart';

typedef OnClickCardCallback = void Function(
    BuildContext context, FolderModel? folderModel, String userCredential);

class GridViewList extends StatelessWidget {

  final FirebaseHandler _firebaseHandler = FirebaseHandler();
  final String _title;
  final String _firebaseCollectionName;
  final String _userCredential;
  final FolderModel? _folder;
  final OnClickCardCallback _onClickAdd;
  final bool _isEditable;

  GridViewList({
    Key? key,
    required String title,
    required String firebaseCollectionName,
    required userCredential,
    required OnClickCardCallback onClickAdd,
    required bool isEditable,
    FolderModel? folder,
  })  : _title = title,
        _firebaseCollectionName = firebaseCollectionName,
        _userCredential = userCredential,
        _folder = folder,
        _onClickAdd = onClickAdd,
        _isEditable = isEditable,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: _title != 'My collections',
        title: Text(_title),
        actions: _buildActions(context),
      ),
      body: _buildGridView(),
    );
  }

  List<IconButton> _buildActions(BuildContext context) {
    if (!_isEditable) {
      return [];
    }
    return [
      IconButton(
        icon: const Icon(Icons.people_outline_outlined),
        onPressed: () {
          Navigator.of(context).push(
              MaterialPageRoute(builder: (_) => FriendsPage(_userCredential)));
        },
      ),
      IconButton(
          icon: const Icon(Icons.add),
          onPressed: () => _onClickAdd(context, _folder, _userCredential)),
      IconButton(
        icon: const Icon(Icons.logout),
        onPressed: () async {
          await _firebaseHandler.signOut();
          await GoogleSignIn().signOut();
          Navigator.of(context).popUntil((route) => route.isFirst);
        },
      ),
    ];
  }

  Widget _buildGridView() {
    final Stream<QuerySnapshot<Map<String, dynamic>>> _stream;
    if (_folder != null) {
      _stream = _firebaseHandler.getFirebaseCollectionByNameUserIdFolderId(
          _firebaseCollectionName, _userCredential, _folder!.id);
    } else {
      _stream = _firebaseHandler.getFirebaseCollectionByNameUserId(
          _firebaseCollectionName, _userCredential);
    }

    return StreamBuilder<QuerySnapshot>(
      stream: _stream,
      builder: (BuildContext context, AsyncSnapshot<QuerySnapshot> snapshot) {
        if (snapshot.hasError) {
          return _buildCenteredText('Something went wrong');
        }

        if (snapshot.connectionState == ConnectionState.waiting) {
          return _buildCenteredText('Loading');
        }

        return Padding(
          padding: const EdgeInsets.all(5.0),
          child: GridView.count(
            crossAxisCount: 2,
            children: snapshot.data!.docs
                .map((DocumentSnapshot document) => _createCard(document))
                .toList(),
          ),
        );
      },
    );
  }

  Widget _createCard(DocumentSnapshot<Object?> document) {
    return GridCard(
        folder: _folder,
        userCredential: _userCredential,
        isEditable: _isEditable,
        document: document);
  }

  Widget _buildCenteredText(String text) {
    return Center(
      child: Text(text),
    );
  }
}
